@@ -45,6 +45,7 @@
 #include "numpy/ndarraytypes.h"
 #include "numpy/npy_3kcompat.h"
 #include "common.h"
+#include "npy_pycompat.h"
 
 #include "arrayobject.h"
 #include "dispatching.h"
@@ -122,8 +123,9 @@ PyUFunc_AddLoop(PyUFuncObject *ufunc, PyObject *info, int ignore_duplicate)
     PyObject *loops = ufunc->_loops;
     Py_ssize_t length = PyList_Size(loops);
     for (Py_ssize_t i = 0; i < length; i++) {
-        PyObject *item = PyList_GetItem(loops, i);
+        PyObject *item = PyList_GetItemRef(loops, i);
         PyObject *cur_DType_tuple = PyTuple_GetItem(item, 0);
+        Py_DECREF(item);
         int cmp = PyObject_RichCompareBool(cur_DType_tuple, DType_tuple, Py_EQ);
         if (cmp < 0) {
             return -1;
@@ -974,7 +976,7 @@ promote_and_get_ufuncimpl(PyUFuncObject *ufunc,
         }
     }
 
-    int current_promotion_state = npy_promotion_state;
+    int current_promotion_state = get_npy_promotion_state();
 
     if (force_legacy_promotion && legacy_promotion_is_possible
             && current_promotion_state == NPY_USE_LEGACY_PROMOTION
@@ -992,10 +994,10 @@ promote_and_get_ufuncimpl(PyUFuncObject *ufunc,
     }
 
     /* Pause warnings and always use "new" path */
-    npy_promotion_state = NPY_USE_WEAK_PROMOTION;
+    set_npy_promotion_state(NPY_USE_WEAK_PROMOTION);
     PyObject *info = promote_and_get_info_and_ufuncimpl(ufunc,
             ops, signature, op_dtypes, legacy_promotion_is_possible);
-    npy_promotion_state = current_promotion_state;
+    set_npy_promotion_state(current_promotion_state);
 
     if (info == NULL) {
         goto handle_error;
@@ -1005,7 +1007,7 @@ promote_and_get_ufuncimpl(PyUFuncObject *ufunc,
     PyObject *all_dtypes = PyTuple_GET_ITEM(info, 0);
 
     /* If necessary, check if the old result would have been different */
-    if (NPY_UNLIKELY(npy_promotion_state == NPY_USE_WEAK_PROMOTION_AND_WARN)
+    if (NPY_UNLIKELY(current_promotion_state == NPY_USE_WEAK_PROMOTION_AND_WARN)
             && (force_legacy_promotion || promoting_pyscalars)
             && npy_give_promotion_warnings()) {
         PyArray_DTypeMeta *check_dtypes[NPY_MAXARGS];
@@ -1014,11 +1016,11 @@ promote_and_get_ufuncimpl(PyUFuncObject *ufunc,
                     all_dtypes, i);
         }
         /* Before calling to the legacy promotion, pretend that is the state: */
-        npy_promotion_state = NPY_USE_LEGACY_PROMOTION;
+        set_npy_promotion_state(NPY_USE_LEGACY_PROMOTION);
         int res = legacy_promote_using_legacy_type_resolver(ufunc,
                 ops, signature, check_dtypes, NULL, NPY_TRUE);
         /* Reset the promotion state: */
-        npy_promotion_state = NPY_USE_WEAK_PROMOTION_AND_WARN;
+        set_npy_promotion_state(NPY_USE_WEAK_PROMOTION_AND_WARN);
         if (res < 0) {
             goto handle_error;
         }
@@ -1070,7 +1072,7 @@ promote_and_get_ufuncimpl(PyUFuncObject *ufunc,
      * then we chain it, because DTypePromotionError effectively means that there
      * is no loop available.  (We failed finding a loop by using promotion.)
      */
-    else if (PyErr_ExceptionMatches(npy_DTypePromotionError)) {
+    else if (PyErr_ExceptionMatches(npy_static_pydata.DTypePromotionError)) {
         PyObject *err_type = NULL, *err_value = NULL, *err_traceback = NULL;
         PyErr_Fetch(&err_type, &err_value, &err_traceback);
         raise_no_loop_found_error(ufunc, (PyObject **)op_dtypes);
@@ -1288,8 +1290,9 @@ get_info_no_cast(PyUFuncObject *ufunc, PyArray_DTypeMeta *op_dtype,
     PyObject *loops = ufunc->_loops;
     Py_ssize_t length = PyList_Size(loops);
     for (Py_ssize_t i = 0; i < length; i++) {
-        PyObject *item = PyList_GetItem(loops, i);
+        PyObject *item = PyList_GetItemRef(loops, i);
         PyObject *cur_DType_tuple = PyTuple_GetItem(item, 0);
+        Py_DECREF(item);
         int cmp = PyObject_RichCompareBool(cur_DType_tuple,
                                            t_dtypes, Py_EQ);
         if (cmp < 0) {
